# -*- coding: utf-8 -*-
"""
Created on Mon Apr 17 18:16:21 2017

@author: Jack C Press
"""

import networkx as nx
import numpy as np
from datetime import datetime
from Preprocessing_Parsing import *

startTime = datetime.now()

#MAKE NETWORKX GRAPH. Length_Graph uses length and Weighted_Length_Graph uses wghtd
Length_Graph = nx.Graph()
Weighted_Length_Graph = nx.Graph()
start = []

for LENGTH, PATH, NODE_ID, WEIGHT in zip(Length, path, ID, wt):
    Length_Graph.add_path(PATH,OID=NODE_ID, weight=LENGTH/(len(PATH)-1))
    Weighted_Length_Graph.add_path(PATH,OID=NODE_ID, weight=WEIGHT/(len(PATH)-1), Length=LENGTH)
#connect missing path near plant
Weighted_Length_Graph.add_path([(lat1, lon1),(lat2, lon2)], weight=50.0, Length=100)

#Set up dictionary which represents node coordinates. IN this case, our nodes ARE coordinates
pos = dict(zip(Length_Graph.nodes(), Length_Graph.nodes()))
Pos = dict(zip(Weighted_Length_Graph.nodes(), Weighted_Length_Graph.nodes()))

#graph nodes with labels 
xnodes = []
ynodes = []
for NODE in nodes:
    xnodes.append(NODE[0])
    ynodes.append(NODE[1])
plt.scatter(xnodes, ynodes)
for i, txt in enumerate(nodeid):
    plt.annotate(txt, (xnodes[i], ynodes[i]), size=5)    
nx.draw_networkx_edges(Weighted_Length_Graph, pos=Pos)

#This is the node which represents the water treatment plant    
Target = WaterTreatmentPlant

#Find Weighted Path Network
WeightedPath = single_source_dijkstra(Weighted_Length_Graph, source=Target)
StationsWeightedPath = single_source_dijkstra(Weighted_Length_Graph, source=Target)


#Set up functions to collect path attributes from weighted path
def path_edge_attributes(PAth):
    return np.sum(list(set([(Weighted_Length_Graph[u][v]['Length']) for (u,v )in zip(PAth[0:],PAth[1:])])))
def path_edge_weight(PAth):
    return (list(set([(Weighted_Length_Graph[u][v]['weight']) for (u,v )in zip(PAth[0:],PAth[1:])])))
def path_edge_wt(PAth):
    return (list([(Weighted_Length_Graph[u][v]['weight']) for (u,v )in zip(PAth[0:],PAth[1:])]))


#Find Each Path's weight, length and ID
Dpath,DnodeX, DnodeY, Ddist, Dweight, DID, Dwt, Dnode = [],[],[],[],[],[],[],[]
for i in range(len(WeightedPath[1])):
    inA = WeightedPath[1][WeightedPath[1].keys()[i]]
    inB = WeightedPath[1].keys()[i]
    inC =  path_edge_attributes(inA)
    Dpath += [inA]
    Dnode += [inB]
    DnodeX += [inB[0]]
    DnodeY += [inB[1]]
    Ddist += [inC]
    Dweight += [path_edge_weight(inA)]
    Dwt += [path_edge_wt(inA)]

#Find the SHORTEST path to Parcel to Plant path by trying EVERY path (computationally EXPENSIVE)    
MP1 = nx.Graph()
MinPathGraph1 = nx.Graph()
MasterPaths1,MPlength1, MasterPaths, MasterMinPath, MasterLengths, MinPath1 = [], [], [], [], [], [] 
AllPaths, MasterMasterMinPaths, MasterMasterLengths, Intersections = = [], [], [], []
MasterIntersections = []
MinPath1 = ()
for m in range(len(sources)):
    MasterLengths = []
    MasterMinPath = []
    Intersections = []
    for i in range(len(Dpath)):
        MasterPaths1 = []
        MPlength1 = []
        MinPath1 = ()
        Min1 = ()
        for b in range(len(Dpath[i])):
            OnePath = dijkstra_path(Weighted_Length_Graph, source=tuple(sources[m]), target=Dpath[i][b], weight='weight')
            MasterPaths1 += [OnePath]
            MP1.add_path(OnePath)
            mplength = path_edge_attributes(OnePath)
            MPlength1 += [mplength]
        AllPaths += [MPlength1]
        MinPath1 = MasterPaths1[MPlength1.index(np.min(MPlength1))]
        MinPathGraph1.add_path(MinPath1)
        MasterLengths += [np.min(MPlength1)]
        MasterPaths += [MasterPaths1]
        MasterMinPath += [MinPath1]
        Intersections += [MinPath1[len(MinPath1)-1]]
    MasterIntersections += [Intersections]
    MasterMasterMinPaths += [MasterMinPath]
    MasterMasterLengths += [MasterLengths]

G = nx.Graph()
for i in range(len(MasterMasterMinPaths)):
    for b in range(len(MasterMasterMinPaths[i])):
        G.add_path(MasterMasterMinPaths[i][b], weight=MasterMasterLengths[i][b])
g = nx.Graph()
for i in range(len(Dpath)):
    g.add_path(Dpath[i])    

#Distance of Monitor/Node Path Intersection to Node  
MPlength2 = []
MP2 = nx.Graph()
NodeToIntersectLengths = []
for m in range(len(MasterIntersections)):
    MPlength2 = []
    for i in range(len(MasterIntersections[m])):
        AnotherPath = dijkstra_path(Weighted_Length_Graph, source=Dnode[i], target=MasterIntersections[m][i], weight='weight')
        MP2.add_path(AnotherPath)
        mplength2 = path_edge_attributes(AnotherPath)
        MPlength2 += [mplength2]
    NodeToIntersectLengths += [MPlength2]
#
print datetime.now() - startTime

#export to CSV file
import csv
DNX = np.transpose(DnodeX)
DNY = np.transpose(DnodeY)
DD = np.transpose(Ddist)
MML = np.transpose(MasterMasterLengths)
NTIL = np.transpose(NodeToIntersectLengths)
with open('DNX.csv','wb') as f:
    w = csv.writer(f)
    w.writerow(DNX)
with open('DNY.csv','wb') as f:
    w = csv.writer(f)
    w.writerow(DNY)
with open('DD.csv','wb') as f:
    w = csv.writer(f)
    w.writerow(DD)
with open('MML.csv','wb') as f:
    w = csv.writer(f)
    for i in range(len(MML)):
        w.writerow(MML[i])
with open('NTIL.csv','wb') as f:
    w = csv.writer(f)
    for i in range(len(NTIL)):
        w.writerow(NTIL[i])
